@@ -6,10 +6,11 @@
 #' \subsection{diseq_directional}{
 #' The directional disequilibrium model consists of three equations and a
 #' separation rule. The market is described by a linear demand, a linear supply
-#' equation and the short side rule. The separation rule splits the sample into regimes
-#' of excess supply and excess demand. If a price change is positive at the time point
-#' of the observation, then the observation is classified as being in an excess demand
-#' regime. Otherwise, it is assumed that it represents an excess supply state. The
+#' equation and the short side rule. The separation rule splits the sample
+#' into states of excess supply and excess demand. If a price change is
+#' positive at the time point of the observation, then the observation is
+#' classified as being in an excess demand state. Otherwise, it is assumed
+#' that it represents an excess supply state. The
 #' model is estimated using full information maximum likelihood.
 #'
 #' \deqn{D_{nt} = X_{d,nt}'\beta_{d} + u_{d,nt},}
@@ -37,8 +38,8 @@ setClass(
 #' # in the directional model prices cannot be included in both demand and supply
 #' model <- new(
 #'   "diseq_directional", # model type
-#'   c("id", "date"), "date", "Q", "P", # keys, time point, quantity, and price variables
-#'   "P + Xd1 + Xd2 + X1 + X2", "Xs1 + X1 + X2", # equation specifications
+#'   subject = id, time = date, quantity = Q, price = P,
+#'   demand = P + Xd1 + Xd2 + X1 + X2, supply = Xs1 + X1 + X2,
 #'   simulated_data, # data
 #'   correlated_shocks = TRUE # allow shocks to be correlated
 #' )
@@ -47,26 +48,23 @@ setClass(
 #' }
 setMethod(
   "initialize", "diseq_directional",
-  function(
-           .Object,
-           key_columns, time_column, quantity_column, price_column,
-           demand_specification, supply_specification,
-           data,
-           correlated_shocks = TRUE, verbose = 0) {
+  function(.Object,
+           quantity, price, demand, supply, subject, time,
+           data, correlated_shocks = TRUE, verbose = 0) {
+    specification <- make_specification(
+      data, quantity, price, demand, supply, subject, time
+    )
     .Object <- callNextMethod(
-      .Object,
-      "Directional", verbose,
-      key_columns, time_column,
-      quantity_column, price_column, demand_specification, supply_specification, NULL,
-      correlated_shocks,
-      data,
+      .Object, "Directional", verbose,
+      specification, correlated_shocks, data,
       function(...) new("system_directional", ...)
     )
 
     # Check for mis-specification
+    price_column <- all.vars(formula(specification, lhs = 2, rhs = 0))
     if (
-      price_column %in% .Object@system@demand@independent_variables &&
-        price_column %in% .Object@system@supply@independent_variables
+      price_column %in% independent_variables(.Object@system@demand) &&
+        price_column %in% independent_variables(.Object@system@supply)
     ) {
       print_error(
         .Object@logger,
@@ -79,13 +77,36 @@ setMethod(
       .Object@logger,
       "Sample separated with ", sum(.Object@system@demand@separation_subset),
       " rows in excess supply and ",
-      sum(.Object@system@supply@separation_subset), " in excess demand regime."
+      sum(.Object@system@supply@separation_subset), " in excess demand state."
     )
 
     .Object
   }
 )
 
+#' @describeIn single_call_estimation Directional disequilibrium model.
+#' @export
+setGeneric(
+  "diseq_directional",
+  function(specification, data,
+           correlated_shocks = TRUE, verbose = 0,
+           estimation_options = list()) {
+    standardGeneric("diseq_directional")
+  }
+)
+
+#' @rdname single_call_estimation
+setMethod(
+  "diseq_directional", signature(specification = "formula"),
+  function(specification, data, correlated_shocks, verbose,
+           estimation_options) {
+    initialize_from_formula(
+      "diseq_directional", specification, data, correlated_shocks, verbose,
+      estimation_options
+    )
+  }
+)
+
 #' @rdname minus_log_likelihood
 setMethod(
   "minus_log_likelihood", signature(object = "diseq_directional"),
