@@ -37,67 +37,48 @@ eq_data <- simulate_data(
 )
 
 ## ----model.parameters---------------------------------------------------------
-key_columns <- c("id", "date")
-time_column <- c("date")
-quantity_column <- "Q"
-price_column <- "P"
-demand_specification <- paste0(price_column, " + Xd1 + Xd2 + X1 + X2")
-supply_specification <- "Xs1 + X1 + X2"
-price_specification <- "Xp1"
 verbose <- 2
 correlated_shocks <- TRUE
-
-## ----model.constructor--------------------------------------------------------
-eqmdl <- new(
-  "equilibrium_model",
-  key_columns,
-  quantity_column, price_column,
-  demand_specification, paste0(price_column, " + ", supply_specification),
-  eq_data[eq_data$date != 1, ],
-  correlated_shocks = correlated_shocks, verbose = verbose
-)
-bsmdl <- new(
-  "diseq_basic",
-  key_columns,
-  quantity_column, price_column,
-  demand_specification, paste0(price_column, " + ", supply_specification),
-  eq_data[eq_data$date != 1, ],
-  correlated_shocks = correlated_shocks, verbose = verbose
-)
-damdl <- new(
-  "diseq_deterministic_adjustment",
-  key_columns, time_column,
-  quantity_column, price_column,
-  demand_specification, paste0(price_column, " + ", supply_specification),
-  eq_data,
-  correlated_shocks = correlated_shocks, verbose = verbose
-)
+formula <-   Q | P | id | date ~ P + Xd1 + Xd2 + X1 + X2 | P + Xs1 + X1 + X2
 
 ## ----estimation.parameters----------------------------------------------------
 optimization_method <- "BFGS"
-optimization_controls <- list(maxit = 10000, reltol = 1e-8)
+optimization_options <- list(maxit = 10000, reltol = 1e-8)
 
-## ----estimation.execution-----------------------------------------------------
-eqmdl_reg <- estimate(eqmdl, method = "2SLS")
-eqmdl_est <- estimate(eqmdl,
-  control = optimization_controls,
-  method = optimization_method
+## ----model.constructor--------------------------------------------------------
+eqmdl_reg <- equilibrium_model(
+  formula, eq_data[eq_data$date != 1, ],
+  correlated_shocks = correlated_shocks, verbose = verbose,
+  estimation_options = list(method = "2SLS")
 )
-bsmdl_est <- estimate(bsmdl,
-  control = optimization_controls,
-  method = optimization_method
+eqmdl_fit <- equilibrium_model(
+  formula, eq_data[eq_data$date != 1, ],
+  correlated_shocks = correlated_shocks, verbose = verbose,
+  estimation_options = list(
+    control = optimization_options, method = optimization_method
+  )
 )
-damdl_est <- estimate(damdl,
-  control = optimization_controls,
-  method = optimization_method
+bsmdl_fit <- diseq_basic(
+  formula, eq_data[eq_data$date != 1, ],
+  correlated_shocks = correlated_shocks, verbose = verbose,
+  estimation_options = list(
+    control = optimization_options, method = optimization_method
+  )
+)
+damdl_fit <- diseq_deterministic_adjustment(
+  formula, eq_data,
+  correlated_shocks = correlated_shocks, verbose = verbose,
+  estimation_options = list(
+    control = optimization_options, method = optimization_method
+  )
 )
 
 ## ----analysis.summaries-------------------------------------------------------
-summary(eqmdl_reg$first_stage_model)
-summary(eqmdl_reg$system_model)
-bbmle::summary(eqmdl_est)
-bbmle::summary(bsmdl_est)
-bbmle::summary(damdl_est)
+summary(eqmdl_reg@fit[[1]]$first_stage_model)
+summary(eqmdl_reg)
+summary(eqmdl_fit)
+summary(bsmdl_fit)
+summary(damdl_fit)
 
 ## ----analysis.estimates-------------------------------------------------------
 sim_coef <- c(
@@ -107,19 +88,13 @@ sim_coef <- c(
   sigma_d, sigma_s,
   rho_ds
 )
-names(sim_coef) <- names(damdl_est@coef)
+names(sim_coef) <- names(coef(damdl_fit))
 
-dm_inc <- eqmdl_reg$system_model$coefficients[
-  grep(
-    "demand",
-    names(eqmdl_reg$system_model$coefficients)
-  )
+dm_inc <- coef(eqmdl_reg)[
+  grep("demand", names(coef(eqmdl_reg)))
 ]
-sp_inc <- eqmdl_reg$system_model$coefficients[
-  grep(
-    "supply",
-    names(eqmdl_reg$system_model$coefficients)
-  )
+sp_inc <- coef(eqmdl_reg)[
+  grep("supply", names(coef(eqmdl_reg)))
 ]
 lm_coef <- c(
   dm_inc[2], dm_inc[-2], sp_inc[2], sp_inc[-2],
@@ -129,18 +104,18 @@ lm_coef <- c(
 )
 
 eqmdl_coef <- append(
-  eqmdl_est@coef, c(NA),
-  after = which(names(eqmdl_est@coef) ==
-    prefixed_variance_variable(eqmdl@system@demand)) - 1
+  coef(eqmdl_fit), c(NA),
+  after = which(names(coef(eqmdl_fit)) ==
+    prefixed_variance_variable(eqmdl_fit@system@demand)) - 1
 )
 
 bsmdl_coef <- append(
-  bsmdl_est@coef, c(NA),
-  after = which(names(bsmdl_est@coef) ==
-    prefixed_variance_variable(bsmdl@system@demand)) - 1
+  coef(bsmdl_fit), c(NA),
+  after = which(names(coef(bsmdl_fit)) ==
+    prefixed_variance_variable(bsmdl_fit@system@demand)) - 1
 )
 
-damdl_coef <- damdl_est@coef
+damdl_coef <- coef(damdl_fit)
 
 comp <- tibble::tibble(
   parameter = names(sim_coef),
@@ -157,20 +132,16 @@ comp_means
 
 ## ----analysis.model.selection-------------------------------------------------
 model_names <- c(
-  eqmdl@model_type_string,
-  bsmdl@model_type_string, damdl@model_type_string
-)
-model_obs <- c(
-  number_of_observations(eqmdl),
-  number_of_observations(bsmdl),
-  number_of_observations(damdl)
+  eqmdl_fit@model_type_string,
+  bsmdl_fit@model_type_string, damdl_fit@model_type_string
 )
+model_obs <- c(nobs(eqmdl_fit), nobs(bsmdl_fit), nobs(damdl_fit))
 model_errors <- c(
   comp_means["fierr"],
   comp_means["bmerr"],
   comp_means["daerr"]
 )
-seltbl <- AIC(eqmdl_est, bsmdl_est, damdl_est) %>%
+seltbl <- AIC(eqmdl_fit, bsmdl_fit, damdl_fit) %>%
   tibble::add_column(Model = model_names, .before = 1) %>%
   tibble::add_column(Obs. = model_obs, `Mean Error` = model_errors) %>%
   dplyr::rename(D.F. = df) %>%
