@@ -11,7 +11,7 @@ parameters <- list(
 # Optimization setup
 reltol <- 1e-8
 optimization_method <- "BFGS"
-optimization_control <- list(REPORT = 10, maxit = 50000, reltol = reltol)
+optimization_options <- list(REPORT = 10, maxit = 50000, reltol = reltol)
 
 # Tests
 mdl <- NULL
@@ -22,35 +22,48 @@ test_that(paste0("Model can be simulated"), {
 
 est <- NULL
 test_that(paste0(model_name(mdl), " can be estimated"), {
-  est <<- estimate(mdl, control = optimization_control, method = optimization_method)
-  expect_is(est, "mle2")
+  est <<- equilibrium_model(
+    formula(mdl), simulated_data,
+    estimation_options = list(
+      control = optimization_options, method = optimization_method
+    )
+  )
+  expect_is(est@fit[[1]], "mle2")
+})
+
+test_that(paste0(model_name(mdl), " fit can be summarized"), {
+  test_summary(est, 41)
 })
 
 test_that(paste0(
   "Maximum likelihood estimates of '", model_name(mdl),
   "' are accurate"
 ), {
-  test_estimation_accuracy(est@coef, unlist(parameters[-c(1, 2)]), 1e-0)
+  test_estimation_accuracy(coef(est), unlist(parameters[-c(1, 2)]), 1e-0)
 })
 
 
 test_that(paste0("Aggregation can be calculated"), {
-  test_aggregation(aggregate_demand, mdl, est@coef)
-  test_aggregation(aggregate_supply, mdl, est@coef)
+  test_aggregation(aggregate_demand, est)
+  test_aggregation(aggregate_supply, est)
 })
 
 test_that(paste0("Scores can be calculated"), {
-  test_scores(mdl, est@coef)
+  test_scores(est)
 })
 
 reg <- NULL
 test_that(paste0("First stage of '", model_name(mdl), "' can be estimated"), {
   reg <<- estimate(mdl, method = "2SLS")
-  expect_is(reg$first_stage_model, "lm")
+  expect_is(reg@fit[[1]]$first_stage_model, "lm")
 })
 
 test_that(paste0("Second stage of '", model_name(mdl), "' can be estimated"), {
-  expect_is(reg$system_model, "systemfit")
+  expect_is(reg@fit[[1]]$system_model, "systemfit")
+})
+
+test_that(paste0(model_name(mdl), " regressions can be summarized"), {
+  test_summary(reg, 75)
 })
 
 test_that(paste0(
@@ -62,7 +75,7 @@ test_that(paste0(
     "beta_s0", "alpha_s", "beta_s", "eta_s"
   )
   test_estimation_accuracy(
-    reg$system_model$coefficients, unlist(parameters[order]), 1e-0
+    coef(reg), unlist(parameters[order]), 1e-0
   )
 })
 
@@ -70,7 +83,8 @@ test_that(paste0("Optimization of '", model_name(mdl), "' using GSL succeeds"),
   mll <<- maximize_log_likelihood(mdl,
     start = NULL, step = 1e-5,
     objective_tolerance = 1e-4,
-    gradient_tolerance = 1e-3
+    gradient_tolerance = 1e-3,
+    max_it = 1e+3
   )
   testthat::expect_length(mll, 8)
 })
@@ -79,5 +93,5 @@ test_that(paste0(
   "Calculated gradient of '", model_name(mdl),
   "' matches the numerical approximation"
 ), {
-  test_calculated_gradient(mdl, est@coef, 1e-4)
+  test_calculated_gradient(mdl, coef(est), 1e-4)
 })
